"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.buildSelector = exports.toSelector = exports.isMatch = exports.clearSelectorCache = void 0;
const cues_1 = require("./cues");
const iterateCues_1 = require("./iterateCues");
const serialize_1 = require("./serialize");
/* eslint-disable @typescript-eslint/no-var-requires */
const { isVisible, querySelectorAll, } = require('playwright-evaluator');
const selectorCache = new Map();
const clickSelectorCache = new Map();
/**
 * @summary Clear the selector cache. Currently only used for tests.
 */
exports.clearSelectorCache = () => {
    selectorCache.clear();
    clickSelectorCache.clear();
};
exports.isMatch = ({ selectorParts, target }) => {
    const result = querySelectorAll({ parts: selectorParts }, document).filter((element) => isVisible(element));
    // console.debug('Try selector', selectorParts[0], selectorParts[1], target);
    if (result[0] !== target && !target.contains(result[0])) {
        // console.error('Selector matches another element');
        return false;
    }
    return true;
};
exports.toSelector = (selectorParts) => {
    const names = selectorParts.map((s) => s.name);
    // CSS selector
    if (!names.includes('text')) {
        return selectorParts.map((s) => s.body).join(' ');
    }
    // mixed selector
    return selectorParts.map(({ body, name }) => `${name}=${body}`).join(' >> ');
};
exports.buildSelector = (options) => {
    const { isClick, target } = options;
    // To save looping, see if we have already figured out a unique
    // selector for this target.
    let cachedSelectorParts;
    if (isClick) {
        cachedSelectorParts = clickSelectorCache.get(target);
    }
    else {
        cachedSelectorParts = selectorCache.get(target);
    }
    let selector;
    if (cachedSelectorParts) {
        // Even if we have cached a selector, it is possible that the DOM
        // has changed since the cached one was built. Confirm it's a match.
        if (exports.isMatch({ selectorParts: cachedSelectorParts, target })) {
            selector = exports.toSelector(cachedSelectorParts);
            // console.debug('Using cached selector', selector, 'for target', target);
            return selector;
        }
    }
    const cues = cues_1.buildCues(options);
    const nearestPreferredAttributeCue = cues.reduce((foundCue, cue) => {
        if (cue.penalty === 0 && (!foundCue || foundCue.level > cue.level)) {
            return cue;
        }
        return foundCue;
    }, null);
    // iterateCues will dynamically figure out increasingly
    // complex cue groups to suggest
    for (let cueGroup of iterateCues_1.iterateCues(cues)) {
        // Because custom attributes are preferred, we assume that at least one (nearest)
        // should ALWAYS be included in every group, even if the selector might be
        // unique without it.
        if (nearestPreferredAttributeCue) {
            if (!cueGroup.find((cue) => cue.penalty === 0)) {
                cueGroup = [...cueGroup, nearestPreferredAttributeCue];
            }
        }
        const selectorParts = cues_1.buildSelectorParts(cueGroup);
        // If the suggested cue group matches this target and no others, use it.
        if (exports.isMatch({ selectorParts, target })) {
            // First cache it so that we don't need to do all the looping for this
            // same target next time. We cache `selectorParts` rather than `selector`
            // because the DOM can change, so when we later use the cached selector,
            // we will need to run it through `isMatch` again, which needs the parsed
            // selector.
            if (isClick) {
                clickSelectorCache.set(target, selectorParts);
            }
            else {
                selectorCache.set(target, selectorParts);
            }
            // Now convert selectorParts (a Playwright thing) to a string selector
            selector = exports.toSelector(selectorParts);
            break;
        }
    }
    // If no selector was unique, fall back to xpath.
    if (!selector) {
        selector = `xpath=${serialize_1.getXpath(target)}`;
    }
    // console.debug('Built selector', selector, 'for target', target);
    return selector;
};
//# sourceMappingURL=selector.js.map